<?php
// Copyright 1999-2015. Parallels IP Holdings GmbH. All Rights Reserved.
class Modules_DropboxBackup_Dropbox
{
    const KEY = 'w70gehtzfpwst5o';
    const SECRET = '9olw4u6mnned4ku';

    /**
     * @var \Dropbox\API[]
     */
    private static $_apiCache = array();

    private static function _getApi($domainId)
    {
        if (!isset(self::$_apiCache[$domainId])) {
            $consumerKey = self::KEY;
            $consumerSecret = self::SECRET;

            $storage = self::_getStorage($domainId);

            if (empty($_SERVER['HTTP_HOST']) || empty($_SERVER['REQUEST_URI'])) {
                $callback = null;
            } else {
                // Check whether to use HTTPS and set the callback URL
                $protocol = isset($_SERVER['HTTPS']) && $_SERVER['HTTPS'] == 'on' ? 'https' : 'http';
                $callback = $protocol . '://' . $_SERVER['HTTP_HOST'] . $_SERVER['REQUEST_URI'];
            }

            try {
                // Create the consumer and API objects
                $OAuth = new \Dropbox\OAuth\Consumer\Curl($consumerKey, $consumerSecret, $storage, $callback);
            } catch (\Dropbox\Exception\NotAuthorizedException $e) {
                if (PHP_SAPI === 'cli') {
                    echo "Need to authenticate with URL: {$e->authorizeUrl}";
                } else {
                    $redirectUrl = Modules_DropboxBackup_Plesk::is121()
                        ? $e->authorizeUrl
                        : pm_Context::getBaseUrl() . "index.php/index/auth?authorizeUrl=" . urlencode($e->authorizeUrl);
                    header("Location: {$redirectUrl}");
                }
                exit;
            }

            self::$_apiCache[$domainId] = new \Dropbox\API($OAuth);
        }
        return self::$_apiCache[$domainId];
    }

    private static function _getStorage($domainId)
    {
        $encryptionKey = Modules_DropboxBackup_Db::getEncryptionKey();
        $encrypter = $encryptionKey ? new \Dropbox\OAuth\Storage\Encrypter($encryptionKey) : null;

        $storage = new \Dropbox\OAuth\Storage\PDO($encrypter, $domainId);
        /** @var \PDO $connection */
        $connection = Modules_DropboxBackup_Db::getDbAdapter()->getConnection();
        $storage->setConnection($connection);
        $storage->setTable('tokens');

        return $storage;
    }

    public static function isAuthorized($domainId)
    {
        $storage = self::_getStorage($domainId);
        return (bool)$storage->get('access_token');
    }

    public static function init()
    {
        // Register a simple autoload function
        spl_autoload_register(
            function ($class) {
                if (0 === strpos($class, 'Dropbox')) {
                    $class = str_replace('\\', '/', $class);
                    require_once(dirname(__FILE__) . '/externals/' . $class . '.php');
                }
            }, true, true
        );
    }

    private $_domainId;

    /** @var \Dropbox\API */
    private $_api;

    public function __construct($domainId)
    {
        $this->_domainId = $domainId;
        $this->_api = self::_getApi($domainId);
    }

    public function getAccount()
    {
        $response = $this->_api->accountInfo();
        if (!isset($response['code']) || $response['code'] != 200) {
            throw new pm_Exception('Failed to Get AccountInfo: ' . print_r($response, true));
        }

        return array('display_name' => $response['body']->display_name, 'email' => $response['body']->email);
    }

    public function getContents()
    {
        $response = $this->_api->metaData('/');
        if (!isset($response['code']) || $response['code'] != 200) {
            throw new pm_Exception('Failed to Get MetaData: ' . print_r($response, true));
        }

        return $response['body']->contents;
    }

    public function getRevisions($file)
    {
        $response = $this->_api->revisions(basename($file));
        if (!isset($response['code']) || $response['code'] != 200) {
            throw new pm_Exception('Failed to Get Revisions: ' . print_r($response, true));
        }

        return $response['body'];
    }

    public function uploadBackup($file)
    {
        $response = $this->_api->chunkedUpload($file);

        if (!isset($response['code']) || $response['code'] != 200) {
            throw new pm_Exception('Failed to Upload: ' . print_r($response, true));
        }
    }

    public function downloadBackup($file, $outFile, $revision)
    {
        $response = $this->_api->getFile(basename($file), $outFile, $revision);

        Modules_DropboxBackup_Log::debug('Dropbox Download Result: ' . print_r($response, true));
    }

    public function logout()
    {
        $storage = self::_getStorage($this->_domainId);
        $storage->delete();
    }
}